@@ -32,22 +32,20 @@ func (r *revision) bumpMinor() {
 	r.minor++
 }
 
-func (r revision) equals(other revision) bool {
-	return r.major == other.major && r.minor == other.minor
-}
-
 func (r revision) compatible(other revision) bool {
 	return r.major == other.major
 }
 
 // Run starts fzf
 func Run(opts *Options) (int, error) {
-	if opts.Tmux != nil && len(os.Getenv("TMUX")) > 0 && opts.Tmux.index >= opts.Height.index {
-		return runTmux(os.Args, opts)
-	}
+	if opts.Filter == nil {
+		if opts.Tmux != nil && len(os.Getenv("TMUX")) > 0 && opts.Tmux.index >= opts.Height.index {
+			return runTmux(os.Args, opts)
+		}
 
-	if needWinpty(opts) {
-		return runWinpty(os.Args, opts)
+		if needWinpty(opts) {
+			return runWinpty(os.Args, opts)
+		}
 	}
 
 	if err := postProcessOptions(opts); err != nil {
@@ -94,11 +92,12 @@ func Run(opts *Options) (int, error) {
 	}
 
 	// Chunk list
+	cache := NewChunkCache()
 	var chunkList *ChunkList
 	var itemIndex int32
 	header := make([]string, 0, opts.HeaderLines)
 	if len(opts.WithNth) == 0 {
-		chunkList = NewChunkList(func(item *Item, data []byte) bool {
+		chunkList = NewChunkList(cache, func(item *Item, data []byte) bool {
 			if len(header) < opts.HeaderLines {
 				header = append(header, byteString(data))
 				eventBox.Set(EvtHeader, header)
@@ -110,7 +109,7 @@ func Run(opts *Options) (int, error) {
 			return true
 		})
 	} else {
-		chunkList = NewChunkList(func(item *Item, data []byte) bool {
+		chunkList = NewChunkList(cache, func(item *Item, data []byte) bool {
 			tokens := Tokenize(byteString(data), opts.Delimiter)
 			if opts.Ansi && opts.Theme.Colored && len(tokens) > 1 {
 				var ansiState *ansiState
@@ -148,13 +147,20 @@ func Run(opts *Options) (int, error) {
 	executor := util.NewExecutor(opts.WithShell)
 
 	// Reader
+	reloadOnStart := opts.reloadOnStart()
 	streamingFilter := opts.Filter != nil && !sort && !opts.Tac && !opts.Sync
 	var reader *Reader
 	if !streamingFilter {
 		reader = NewReader(func(data []byte) bool {
 			return chunkList.Push(data)
 		}, eventBox, executor, opts.ReadZero, opts.Filter == nil)
-		go reader.ReadSource(opts.Input, opts.WalkerRoot, opts.WalkerOpts, opts.WalkerSkip)
+
+		if reloadOnStart {
+			// reload or reload-sync action is bound to 'start' event, no need to start the reader
+			eventBox.Set(EvtReadNone, nil)
+		} else {
+			go reader.ReadSource(opts.Input, opts.WalkerRoot, opts.WalkerOpts, opts.WalkerSkip)
+		}
 	}
 
 	// Matcher
@@ -170,7 +176,6 @@ func Run(opts *Options) (int, error) {
 			forward = true
 		}
 	}
-	cache := NewChunkCache()
 	patternCache := make(map[string]*Pattern)
 	patternBuilder := func(runes []rune) *Pattern {
 		return BuildPattern(cache, patternCache,
@@ -229,7 +234,8 @@ func Run(opts *Options) (int, error) {
 	}
 
 	// Synchronous search
-	if opts.Sync {
+	sync := opts.Sync && !reloadOnStart
+	if sync {
 		eventBox.Unwatch(EvtReadNew)
 		eventBox.WaitFor(EvtReadFin)
 	}
@@ -249,7 +255,7 @@ func Run(opts *Options) (int, error) {
 	if heightUnknown {
 		maxFit, padHeight = terminal.MaxFitAndPad()
 	}
-	deferred := opts.Select1 || opts.Exit0
+	deferred := opts.Select1 || opts.Exit0 || sync
 	go terminal.Loop()
 	if !deferred && !heightUnknown {
 		// Start right away
@@ -316,6 +322,9 @@ func Run(opts *Options) (int, error) {
 					err = quitSignal.err
 					stop = true
 					return
+				case EvtReadNone:
+					reading = false
+					terminal.UpdateCount(0, false, nil)
 				case EvtReadNew, EvtReadFin:
 					if evt == EvtReadFin && nextCommand != nil {
 						restart(*nextCommand, nextEnviron)
@@ -341,9 +350,6 @@ func Run(opts *Options) (int, error) {
 					}
 					total = count
 					terminal.UpdateCount(total, !reading, value.(*string))
-					if opts.Sync {
-						terminal.UpdateList(PassMerger(&snapshot, opts.Tac, snapshotRevision), false)
-					}
 					if heightUnknown && !deferred {
 						determine(!reading)
 					}
@@ -431,7 +437,7 @@ func Run(opts *Options) (int, error) {
 								determine(val.final)
 							}
 						}
-						terminal.UpdateList(val, true)
+						terminal.UpdateList(val)
 					}
 				}
 			}
